/*********************************************************************************
 * capifax.c
 *
 * Copyright (c) 2008-2009 Marco Zissen <maz@v3v.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *********************************************************************************/
#include "capifax.h"

int main(int argc, char *arg[])
{
    int ret = -1;
    
    printf("-------------------------------------------------------------------------------\n");    
    printf(" %s %s %s\n", APPNAME, APPVERSION, APPCR);
    printf("-------------------------------------------------------------------------------\n");
    
    // Parameter
    if(param_parser(argc, arg)) {
        show_usage();
        return 0;
    }
        
    // Init signal handler (CTRL+C / Terminate)
    signal(SIGINT,  signal_handler);
    signal(SIGTERM, signal_handler);

    // Initialize
    ret = capifax_init(m_param.controller, m_param.loglevel_lib, &status, &progress, NULL, NULL);    
    
    if(ret == 0)
    {    
        char *header = NULL;
        char *ident = NULL; 
        
        if(strlen(m_param.header)) header = &m_param.header[0];
        if(strlen(m_param.ident)) ident = &m_param.ident[0];
        
        m_param.success = 0;

        if(!m_param.sending) 
        {
            while(!exit_loop)
            {    
                char tmp_file[512];
                char new_file[512];
                char trg_no[64];    

                memset(trg_no, 0, sizeof(trg_no));
                
                #ifdef WIN32
                snprintf(tmp_file, sizeof(tmp_file), "%s\\capifax_%s.tiff", m_param.tiff_file, m_param.src_no);
                #else
                snprintf(tmp_file, sizeof(tmp_file), "%s/capifax_%s.tiff", m_param.tiff_file, m_param.src_no);
                #endif

                // receive fax                
                ret = capifax_recv(tmp_file, m_param.rate, m_param.ecm, m_param.src_no, trg_no, ident, header, m_param.manual);
                
                // rename file                
                {
                    time_t tim = time(NULL);
                    struct tm *now = localtime(&tim);        
                            
                    #ifdef WIN32
                    snprintf(new_file, sizeof(new_file), "%s\\capifax_%s_%s-%02d_%02d_%d_%02d_%02d_%02d.tiff", m_param.tiff_file, m_param.src_no, trg_no, now->tm_mday, now->tm_mon+1, now->tm_year+1900, now->tm_hour, now->tm_min, now->tm_sec);
                    #else
                    snprintf(new_file, sizeof(new_file), "%s/capifax_%s_%s-%02d_%02d_%d_%02d_%02d_%02d.tiff", m_param.tiff_file, m_param.src_no, trg_no, now->tm_mday, now->tm_mon+1, now->tm_year+1900, now->tm_hour, now->tm_min, now->tm_sec);
                    #endif
                    rename(tmp_file, new_file);

                    if(m_param.loglevel_app == 1) {
                        if(m_param.success) {
                            printf(" %s\n", new_file);                        
                            printf("-------------------------------------------------------------------------------\n");
                        }                        
                    }
                    
                    m_param.success = 0;
                }

                // receive single fax?
                if(m_param.single)
                    break;
            }
        } 
        else 
        {
            // send fax
            ret = capifax_send(m_param.tiff_file, m_param.rate, m_param.ecm, m_param.controller, m_param.src_no, m_param.trg_no, ident, header, 0);    
            if(m_param.success) ret = 0;
        }
    } else {
        printf(" Error %d: CAPI initialisation failed!", ret);
    }
    
    // not interrupted by signal handler?
    if(!exit_loop)
        capifax_close();

    // wait, until signal handler completed
    while(exit_loop == 1)
        usleep(25);

    printf(" capifax exited!\r\n\r\n");

    return ret;
}

void signal_handler(int sig_num)
{
    printf(" Received signal. Exiting now...\r\n");

    exit_loop = 1;    
    capifax_close();    
    exit_loop = 2;
}

void STDCALL progress(int con, capifax_status_t *progress)
{   
    /*
    if(m_param.sending)     
        printf(" Bytes:          %d\n", progress->bytes_sent);        
    else      
        printf(" Bytes:          %d\n", progress->bytes_total);            
    */
}

void STDCALL status(int con, capifax_status_t *status)
{    
    if(m_param.loglevel_app == 1) 
    {        
        // Initialize
        if(status->phase == CONNECT) {
            if(m_param.sending) printf(" OUTGOING FAX\n");
            else printf(" INCOMING FAX\n");
            m_param.success = 0;
        }
        
        // Start receiving / sending
        if(status->phase == PHASE_B) {
            if(m_param.sending) printf(" SENDING FAX (START)\n");
            else  printf(" RECEIVING FAX (START)\n");
        }

        // Page
        if(status->phase == PHASE_D) {
            if(m_param.sending) printf(" SENDING FAX (PAGE)\n");
            else  printf(" RECEIVING FAX (PAGE)\n");
        }

        // End-Of-Fax
        if(status->phase == PHASE_E) {
            if(m_param.sending) printf(" SENDING FAX (END)\n");
            else  printf(" RECEIVING FAX (END)\n");
        }

        // Successful
        if(status->phase == FAX_OK) {
            if(m_param.sending) printf(" SENDING FAX (OK)\n");
            else  printf(" RECEIVING FAX (OK)\n");
            m_param.success = 1;
        }

        // Failed!
        if(status->phase == FAX_FAILED) {
            if(m_param.sending) printf(" SENDING FAX (FAILED %d)\n", status->error_code);
            else  printf(" RECEIVING FAX (FAILED %d)\n", status->error_code);
            m_param.success = 0;
        }

        // Rejected!
        if(status->phase == FAX_REJECTED) {
            if(m_param.sending) printf(" SENDING FAX (REJECTED)\n");
            else  printf(" RECEIVING FAX (REJECTED)\n");
            m_param.success = 0;
        }
        printf(" Phase:          %d\n", status->phase);
        printf(" Connection:     %d\n", con);        
                       
        if(status->phase != 1 && status->phase != 7) 
        {
            printf(" Bitrate:        %d\n", status->bitrate);
            printf(" Encoding:       %d\n", status->encoding);
            printf(" Bad rows:       %d\n", status->bad_rows);
            printf(" Page:           %d / %d\n", status->page_current, status->page_total);   
        } 
        else 
        {
            printf(" Src-No:         %s\n", status->src_no);
            printf(" Dst-No:         %s\n", status->trg_no);
        }

        // Ask for confirm...
        if(status->phase == 1 && !status->sending && status->manual_hookup)
        {
	    char buf[256];
            char ask;
	    	    
            printf("-------------------------------------------------------------------------------\n");
            printf(" Accept incoming connection [y/n]? ");
	    	    
	    do {
		fgets(buf, sizeof(buf)-1, stdin);
    		sscanf(buf, "%c", &ask);
	    } while(ask != 'Y' && ask != 'y' && ask != 'N' && ask != 'n');
	    
	    if(ask == 'y' || ask == 'Y')
                capifax_accept(con);
            else
                capifax_reject(con);
	
	    fflush(stdin);
        }

        printf("-------------------------------------------------------------------------------\n");
    }    
}

static void show_usage()
{    
    printf(" usage: %s [OPTIONS]\n\n", APPNAME);
    
    printf(" OPTIONS:\n");
    printf("  -send     [tiff_file] [controller] [msn] [target_no]\n");
    printf("  -receive  [path] [controller] [msn]\n");    
    printf("  -rate     [2400|4800|9600|14400]\n");
    printf("  -ident    \"+00 00 0000000\"\n");
    printf("  -header   \"My Name\"\n");
    printf("  -ecm      [on|off]\n");
    printf("  -manual   [on|off]\n"); 
    printf("  -single   [on|off]\n");
    printf("  -loglevel [0-5]\n");
    printf("\n");
}

static int param_parser(int argc, char *arg[])
{
    int ret = 1;
    int i;
    char tmp[32];
    
    memset(&m_param, 0, sizeof(m_param));
    m_param.ecm = 1;
    m_param.controller = 1;
    m_param.rate = 3;
    m_param.single = 0;
    m_param.manual = 0;
    m_param.loglevel_app = 1;
    m_param.loglevel_lib = 0;        
    snprintf(m_param.ident, sizeof(m_param.ident), "+00 00 0000000");
    
    for(i=0; i<argc; i++)
    {
        // -send
        if(!strcmp(arg[i], "-send") && i+4 < argc)
        {
            snprintf(tmp, sizeof(tmp), "%c", arg[i+2][0]);            
            tmp[1] = '\0';
            if(isdigit(tmp[0]))
            {                
                snprintf(m_param.tiff_file, sizeof(m_param.tiff_file), "%s", arg[i+1]);
                m_param.controller = atoi(tmp);
                snprintf(m_param.src_no, sizeof(m_param.src_no), "%s", arg[i+3]);
                snprintf(m_param.trg_no, sizeof(m_param.trg_no), "%s", arg[i+4]);
                m_param.sending = 1;
                ret = 0;
            }
        }
        
        // -receive
        if(!strcmp(arg[i], "-receive") && i+3 < argc)
        {
            snprintf(tmp, sizeof(tmp), "%c", arg[i+2][0]);
            tmp[1] = '\0';
            if(isdigit(tmp[0]) || !strcmp(tmp, "-1"))
            {
                snprintf(m_param.tiff_file, sizeof(m_param.tiff_file), "%s", arg[i+1]);                
                m_param.controller = atoi(tmp);
                snprintf(m_param.src_no, sizeof(m_param.src_no), "%s", arg[i+3]);
                m_param.sending = 0;
                ret = 0;
            }            
        }
        
        // -manual
        if(!strcmp(arg[i], "-manual") && i+1 < argc)        
        {
            if(!strcmp(arg[i+1], "on")) m_param.manual = 1;
            else m_param.manual = 0;
        }

        // -single
        if(!strcmp(arg[i], "-single") && i+1 < argc)        
        {
            if(!strcmp(arg[i+1], "on")) m_param.single = 1;
            else m_param.single = 0;
        }

        // -rate
        if(!strcmp(arg[i], "-rate") && i+1 < argc)        
        {
            if(!strcmp(arg[i+1], "2400")) m_param.rate = 0;
            if(!strcmp(arg[i+1], "4800")) m_param.rate = 1;
            if(!strcmp(arg[i+1], "9600")) m_param.rate = 2;
            if(!strcmp(arg[i+1], "14400")) m_param.rate = 3;
        }
        
        // -header
        if(!strcmp(arg[i], "-header") && i+1 < argc)        
            snprintf(m_param.header, sizeof(m_param.header), "%s", arg[i+1]);
        
        // -ident
        if(!strcmp(arg[i], "-ident") && i+1 < argc)        
            snprintf(m_param.ident, sizeof(m_param.ident), "%s", arg[i+1]);
        
        // -ecm
        if(!strcmp(arg[i], "-ecm") && i+1 < argc)
        {
            if(!strcmp(arg[i+1], "on")) m_param.ecm = TRUE;
            else m_param.ecm = FALSE;
        }
        
        // -loglevel
        if(!strcmp(arg[i], "-loglevel") && i+1 < argc)
        {
            int loglevel = 0;
            snprintf(tmp, sizeof(tmp), "%c", arg[i+1][0]);
            tmp[1] = '\0';
            
            if(isdigit(tmp[0])) loglevel = atoi(tmp);            

            if(loglevel != 0) m_param.loglevel_app = 1;   // App-Loglevel (1)
            if(loglevel == 2) m_param.loglevel_lib = 1;   // Lib-Loglevel (2-5 to 1-4)
            if(loglevel == 3) m_param.loglevel_lib = 2;
            if(loglevel == 4) m_param.loglevel_lib = 3;
            if(loglevel > 4) m_param.loglevel_lib = 4;
        }
    }
    
    return ret;
}
